/* Datei:  comsf_lcd.c
 * Author: gkemnitz
 * Datum:  09.11.2016
 *
 * Quellprogramm für den Gerätetreiber comsf_lcd
 *
 * PModLS an JD oben
 */ 

#include "comsf_lcd.h"

// globale private Daten
uint8_t LCD_dat[32];  // Ausgabetext
uint8_t lcd_idx;      // Indexvariable

void lcd_init(uint8_t *text){ // LCD-Treiber initialisieren
  // USART1 mit 8N1, 9600 Baud initialisieren
  UCSR1C = 0b110;             // Übertragungsformat 8N1
  UBRR1 = 51;                 // 9600 Baud
  UCSR1B |= (1<<TXEN1);	      // Sender ein

  // 8 Zeichen LC-Display-Initialisierung 
  uint8_t lcd_init_dat[] = "\x1B[0h\x1B[j";
  for (lcd_idx=0; lcd_idx<7; lcd_idx++){
    while (!(UCSR1A&(1<<UDRE1)));
    UDR1 = lcd_init_dat[lcd_idx];
  }
  // Initialisierung des Hintergrundtextes
  for (lcd_idx=0; lcd_idx<32; lcd_idx++) // Übernahme des
    LCD_dat[lcd_idx] = text[lcd_idx];    // Init.-Texts
  lcd_idx = 0;                           // Index auf Feldanfang
}

void lcd_step(){			 // Schrittfunktion
  if (UCSR1A&(1<<UDRE1)){                // wenn Puffer frei
    UDR1 = LCD_dat[lcd_idx];             // schicke nächstes 
    lcd_idx++;                           // Zeichen
    if (lcd_idx>=32) lcd_idx = 0;
  }                                      // nach dem letzten
}                                        // folgt erstes Zeichen

// Saettigungszähler auf LCD erhöhen
// Reihenfolge ->1->2->..->9->a->b->c->..->z
void lcd_incErr(uint8_t pos){
	pos &= 0x1F;                     // WB auf 0:31 begrenzen
	uint8_t w = LCD_dat[pos];
	if (w>='0' && w<'9') w++;
	else if (w=='9') w = 'a';
	else if (w>='a' && w<'z') w++;
	else if (w=='z');
	else w = '1';
	LCD_dat[pos] = w;
}

// Zeichen auf LCD ausgeben
void lcd_disp_chr(uint8_t c, uint8_t pos){
  LCD_dat[pos & 0x1F] = c;     // Position mod-32
}                              // begrenzen und Zeichen schreiben

// Text auf LCD ausgeben
void lcd_disp_str(uint8_t *str, uint8_t pos, uint8_t len){
  while (len){                 // für alle Zeichen
    lcd_disp_chr(*str, pos);   // Zeichen schreiben
    str++; pos++; len--;       //
  }
}

// Zahl auf LCD ausgeben
void lcd_disp_val(uint32_t val, uint8_t pos, uint8_t len){
  while (len){
    len--;                     // Ziffernanzahl verringern
    lcd_disp_chr((val % 10) + '0', pos+len); // Rest mod 10
    val = val / 10;            // Ziffer und Wert durch 10
  }
  if (val)                     // wenn Stellenzahl zu klein
  lcd_disp_chr('?', pos);      // Ersatz 1. Ziffer durch '?'
}
