/* Datei:  test_comir.c
 * Autor: gkemnitz
 * letzte Änderung:  02.11.2015
 *
 * Beispiel zu Foliensatz IW4
 * Testprogramm für die Teiber
 * comir_pc, comir_sonar und comir_lcd
 *
 * PModLS an JD oben, Sonar an JD unten
 * PmodUSBUSART an JH oben
 * JDX und JHX "gekreuzt (=)"
 */

#include "comir_pc.h"
#include "comir_sonar.h"
#include "comir_lcd.h"
#include "comir_tmr.h"
#include <avr/interrupt.h>

#define INITSTR "S:... I:.... L:.T:... W:. E:...."
#define LCP_SONAR      2 // Sonarwert
#define LCP_RMSG       8 // Eingabedaten
#define LCP_LBYTE     15 // letztes empf. Byte
#define LCP_TIME      18 // Zeitanzeige
#define LCP_WAIT      24 // Wartezeit
//Anzeigepositionen für Fehlerzähler
#define ERR_SEND      28 // Sendeversagen
#define ERR_ETO       29 // Empfangstimout
#define ERR_TEST      30 // zählt Empfangsnachrichten
// Zeichen 31 ist der Zähler falsche Interrupts

uint8_t mrmsg[COM_PC_RMSG_LEN]; // Empfangsnachricht
uint8_t msmsg[COM_PC_SMSG_LEN]; // Sendnachricht


int main(void){
	uint8_t state='E';    // Programmzustand (E-V-A)
	uint16_t snrval;      // Sonarwert
	uint8_t sn_ct, msg_ct;// Sonarwert und Nachrichtenz.
	sonar_init();         // alle Hintergrundprozesse
	com_pc_init();        // initialisieren
	lcd_init((uint8_t*)INITSTR);
	tmr_init();
  // nicht behandelter Interrupt ca. alle 8 s
  TCCR4B = 0b101;      // Timer 4, Nomalmodus, Vorteiler
  TIMSK4 = 1<<TOIE4;   // 1024, Überlauf-Interrupt ein

	sei();                        // Interrupts freigeben
	while(1) {
		if (sonar_get(&snrval)){    // wenn neue Sonardaten
      // Sonarwert auf LCD ausgeben
			lcd_disp_val(snrval, LCP_SONAR, 3);
			sn_ct++;                  // Sonarwerte zählen
		}
		if (state=='E'){            // wenn Programmz. "Eingabe"
			if (com_pc_get(mrmsg)){   // Wenn neue PC-Nachricht
				// diese übernehmen, auf LCD ausgeben
				lcd_disp_str(mrmsg, LCP_RMSG, COM_PC_RMSG_LEN);
				lcd_incErr(ERR_TEST);   // Testfehlerzähler erhöhen
				msg_ct++;
				tmr_start(80, 0);       // Tmr-Kan. 0 für 8s starten
				state = 'V';            // Programmzust. "Verarbeitung"
			}
		}
		else if (state=='V'){
			// Ausgabe Wartezeit bis zur nächsten EA-Operation
			lcd_disp_val(tmr_restzeit(0)/10, LCP_WAIT, 1);
			if (!tmr_restzeit(0))
			state = 'A';             // Programmzustand "Ausgabe"
		}
		else{	                     // wenn Programmz. "Ausgabe"
			msmsg[0] = snrval >> 8;  // Sensor- und Zählwerte
			msmsg[1] = snrval & 0xFF;// byteweise in die Send-
			msmsg[2] = sn_ct;        // nachricht schreiben
			msmsg[3] = msg_ct;
			if (!com_pc_send(msmsg)) // "string" versenden, wenn
			  lcd_incErr(ERR_SEND);  // erfolglos, Fehlerz. ++
			state = 'E';             // Programmzustand "Eingabe"
		}
		// immer letztes empf. Byte auf LCD schreiben
		lcd_disp_chr(com_pc_last_byte(), LCP_LBYTE);
		// Zeit seit Programmstart in s ausgeben
		lcd_disp_val((tmr_get()/10) % 1000, LCP_TIME, 3);
		if (com_pc_err())          // Wenn Empfangs-Timeout
		  lcd_incErr(ERR_ETO);     // Fehlerzähler erhöhen
	}
}
