/* Datei: TestAbstandssensor.c
 * Autor: gkemnitz
 * letzte nderung: 02.11.2015
 * 
 * Testprogramm fr den IR-Abstandssensor
 *
 * PModLS an JD oben 
 * Sharp-Abstandssensor an ADC0 (PF0) 
 * (schwarz: Gnd, rot: 5V, gelb: Signal)
 */ 

#include <avr/io.h>
#include "comir_lcd.h"
#include <avr/interrupt.h>

void adc_init(){
  ADMUX = 0b01000000; // ADC0 (PF0) mit AREF = AVCC
  // Einschalten mit Wandlungstaktteiler 64
  ADCSRA = (1<<ADEN)|(0b110<<ADPS0); 
  DDRF &= ~0x01;     // Sensoreingang als Eingang
  PORTF &=~0x01;     // Ausgabewert 0 (hochohmig)
}

uint32_t get_adc(){
  uint16_t i;
  uint32_t wert=0;
  // U=3.3V*W/1024, 322 Messungen => Wert in
  for (i=0;i<322;i++){    // in 10V-Schritten
    ADCSRA |= (1<<ADSC);  //Wandlung starten
    // Warten bis "fertig"-Bit gesetzt ist
    while(!(ADCSRA & (1<<ADIF)));
                          // "fertig"-Bit lschen 
    ADCSRA |= (1<<ADIF);  // durch 1 Schreiben
    wert += ADC;
  }
  return wert;
}

// Definition der Ausgabedarstellung fr das LCD
#define INITSTR "U_abst: .... mV Ct:......       "
#define LCP_UABST   8     // Sensorspannung
#define LCP_Ct     19     // Zhler


int main(void){
  adc_init();	          // ADC Kanal 0  initialisieren
  lcd_init((uint8_t*)INITSTR);
  uint32_t dat, Ct=0;     // Messwert und Datenzhler
  sei();
  while(1) {
    dat = get_adc();
    lcd_disp_val(dat/100, LCP_UABST, 4);
    lcd_disp_val(Ct++,LCP_Ct, 6);
  }
}
