# Datei: rotmess.py (Python 3)
# Autor: gkemnitz
# letzte Aenderung: 28.11.2016
#
# PC-Steuerprogramm zur Messung der Motorkennlinie
#
# Bestimmung der Rotationsgeschwindigkeit in Winkelschritten pro Sekunde
# fuer die PWM-Perioden 2ms, 1ms und 0,5ms und die relativen Pulsbreiten 
# -100% bis +100%. (aus dem Vorzeichen wird das Richtungssignal). 
# Tabellarische und graphische Ergebnisdarstellung.
# 
# Auf dem Mikrorechner muss das Programm "test_rotmess" laufen.
# Als COM-Schnittstelle die der seriellen Verbindung einstellen.


import serial                   # Modul serial importieren
ser = serial.Serial("COM5")     # Die zu oeffnende serielle 
                                # ist Schnittstelle anzupassen
import matplotlib.pyplot as plt # Plotfunktion importieren
                                # wiederhole fuer 4 Periodenwerte
Geriebeuntersetzung = 19 #19 für 12V und 53 für 6V-Motoren 
								
for Periode in (0x4000,0x2000,0x1000):
  pwm_list=[]
  speed_list=[]
  pwm=-1.0
  while pwm<=1.01:            # wiederhole fuer pwm-Werte von -1 bis 1
    ocr = int(pwm*Periode)    # Umwandlung der %-Zahl in einen OCR-Wert
    bocr = ocr.to_bytes(2, byteorder='big', signed=True) # OCR-Wert zu Byte-Array, Länge 2, signed
    bper = Periode.to_bytes(2, byteorder='big')          # Periode zu Byte-Array, Länge 2, unsigned
    smsg = bocr + bocr + bper # Bytearray von 6 Bytes: 2x OCR (Motor R+L), 1x Periode
    ser.write(smsg)           # senden von 6 Bytes
    rmsg = ser.read(8)        # auf 8 Bytes warten
    # Bytevektor in Ergebnisswerte umrechnen und ausgeben
    pwmR = float(int.from_bytes(rmsg[0:2], byteorder='big', signed=True))/Periode * 100
    spR  = float(int.from_bytes(rmsg[2:4], byteorder='big', signed=True))/(12*Geriebeuntersetzung)
    pwmL = float(int.from_bytes(rmsg[4:6], byteorder='big', signed=True))/Periode * 100
    spL  = float(int.from_bytes(rmsg[6:8], byteorder='big', signed=True))/(12*Geriebeuntersetzung)
    print('Periode = %4.2fms'%(Periode/8E3), 'pwmR =%6.1f'%pwmR + '%', 'spR = %4.2f U/s'%spR, ' | ', 'spL = %4.2f U/s'%spL)
    # Ergebnisse fuer Motor A zusaetzlich an Listen haengen 
    pwm_list.append(pwm)
    speed_list.append(spR)
    pwm +=0.05                 # Erhoehung in 5% Schritten
  # Liste als xy-Graph ausgeben, einen je Periode
  plt.plot(pwm_list, speed_list)

ser.close()                    # serielle Verbindung schliessen

plt.xlabel('pwm in %')         # Achsen beschriften
plt.ylabel('Umdrehungen pro s')
plt.show()
fig = plt.figure()
savefig(tmp.fig)
